// *******************************************************************
//    IBM Rational Ltd  Rhapsody in C++ Simplified eXecution Framework
//   ================== ==============================================
//
//    Model Element   : OMMainThread
//    File Path       : OMMainThread.cpp 
//    Licensed Materials - Property of IBM
//    (c) Copyright IBM Corporation 2008, 2016. All Rights Reserved.
//    
//    Note to U.S. Government Users Restricted Rights:  
//    Use, duplication or disclosure restricted by GSA ADP Schedule 
//    Contract with IBM Corp.
// *******************************************************************


#include "OMMainThread.h"
#include "GuardMacros.h"
#include "OMEvent.h"
#include "OMOSSpecific.h"
#include "OMReactive.h"
#include "OMTimeout.h"
//## package SXF::OMGeneric_Pkg

//## class OMMainThread
//#[ requirement(s)
// Realizes requirement Main Thread #SXF_LR.1.3:
// OMMainThread singleton class shall provide main thread and timeout pool handling.
// It shall inherite from OMThread class.
//#]
//#[ requirement(s)
// Realizes requirement MainThread Constructor #SXF_LR.1.3.3:
// Constructor of MainThread shall initialize timeout pool and list of active threads. 
// It shall be private to avoid duplication.
//#]
OMMainThread::OMMainThread(void) : OMThread(NULL) {
    //#[ operation OMMainThread()
    //#]
}

//#[ requirement(s)
// Realizes requirement Access toTimeout Pool #SXF_LR.1.3.4:
// This function shall return pointer to timeout pool.
// It shall be public.
//#]
OMTimeoutPool* OMMainThread::getItsTimeoutPool(void) {
    //#[ operation getItsTimeoutPool()
    return &itsTimeoutPool;
    //#]
}

//#[ requirement(s)
// Realizes requirement Timeout Processing #SXF_LR.1.3.1:
// It shall process all expired timeouts in timeout pool and put them into event queue of its reactive object. 
// If queuing timeout event fails, it shall  call callback(virtual function) to give user opportunity to process this situation.
// It shall be public and virtual.
//#]
void OMMainThread::postTimeouts(void) {
    //#[ operation postTimeouts()
    OMTimeout* aTimeout = itsTimeoutPool.getExpiredTimeout();
    //all expired timeouts should be put into appropriate event queue
    while (aTimeout != NULL ) {
    	// Add expired Timeout to the event queue
    	OMReactive* dest = aTimeout->getDestination();
    	if (dest)
    	{
    		OMThread* threadP = dest->getActiveContext() ;
    		if (threadP)
    		{
    			const bool retCode = threadP->queueEvent( aTimeout );
    			if (!retCode)
    			{   
    				dest->handleTmEventNotQueued(aTimeout);
    			}
    		}
    	}
    	aTimeout = itsTimeoutPool.getExpiredTimeout();
    }
    //#]
}

//#[ requirement(s)
// Realizes requirement Timeout Pool initialization #SXF_LR.1.3.5:
// The function OMMainThread::setTimeoutPool has two parameters: 
// - the first one is a 'pointer to first element of an array' of type OMTimeout*,
// - the second one is a 'number of elements of the array') of type Rhp_uint32_t.
// This function shall save address of statically declared array of timeouts (the first parameter)
// and initialize link list of free timeout elements with all elements of this array such that the following is true (let n be the number of array-elements):
// - For all 0<=i<n-1, the array element with index i points to the array element with the index i+1 via their 'itsNextTimeout'-attribute.
// - The itsNextTimeout of the last array element (with index n-1) is NULL 
// The array of timeouts shall be declared in generated main function of application.
// The itsOMTimeoutPool of all timeouts in the array refers to the OMTimeoutPool referenced by the OMMainThread::itsTimeoutPool attribute.
// The function shall be public.
//#]
/*LDRA_INSPECTED 120 D 1 : MISRA-C++:2008 7-1-2: "timeoutPool" parameter cannot be const */
void OMMainThread::setTimeoutPool(OMTimeout* const arrayOfTimeouts, const Rhp_uint32_t timeoutPoolSize) {
    //#[ operation setTimeoutPool(OMTimeout,Rhp_uint32_t)
    Rhp_uint32_t lastInd=0U;
    // link list is created
    OMTimeout*  arrayOfTimeoutsPtr = arrayOfTimeouts;
    for (Rhp_uint32_t i=0U;  i < (timeoutPoolSize-1U); i++) {
    	arrayOfTimeoutsPtr[i].initTimeout(&arrayOfTimeoutsPtr[i+1U],&itsTimeoutPool);
    	lastInd++;	
    }
    // The last element of timeout pool has NULL "next" 
    arrayOfTimeoutsPtr[lastInd].initTimeout(NULL,&itsTimeoutPool);	 
    // Head of link list is put into itsFreeList
    itsTimeoutPool.setItsFreeList(arrayOfTimeouts);
    
    //#]
}

//#[ requirement(s)
// Realizes requirement Get MainThread Instance #SXF_LR.1.3.2:
// This function  shall return pointer to main thread instance, which shall be statically defined inside of the function.
// It shall be public and static.
//#]
OMMainThread* OMMainThread::theThread(void) {
    //#[ operation theThread()
    static OMMainThread instance;
    return &instance;
    //#]
}

/*********************************************************************
	File Path	: ../OMMainThread.cpp
*********************************************************************/
