@rem ***************************************************************************
@rem Licensed Materials - Property of IBM
@rem (c) Copyright IBM Corporation 2018. All Rights Reserved.
@rem
@rem Note to U.S. Government Users Restricted Rights: 
@rem Use, duplication or disclosure restricted by GSA ADP Schedule 
@rem Contract with IBM Corp.
@rem ***************************************************************************
@echo off

echo ****************** Rhapsody iFix Post Installation ********************
setlocal EnableDelayedExpansion

echo Script is running from:
echo 	"%cd%"

if [%3]=="FromInstaller" (
	set FromInstaller=TRUE
)

if [%1]==[] (
	echo Usage: post_install ^<Rhapsody architecture:32bit/64bit^> ^<Rhapsody version: e.g. 8.3.1^>
	goto finish
)

set ARCH=%1
set VER=%2

rem rhapsody common dir is under wow6432Node only if system is 64 bit and rhapsody installation was 32 bit

set RegQry=HKLM\Hardware\Description\System\CentralProcessor\0
%SystemRoot%\System32\REG.exe Query %RegQry% | %SystemRoot%\System32\FindSTR /C:"x86" >nul & IF  ERRORLEVEL 1 (
	set OS_ARCH=x64
) else (
	set OS_ARCH=x86
)
rem echo OS Arhitecture is %OS_ARCH%

set COMMONREGKEY="HKEY_LOCAL_MACHINE\SOFTWARE\Telelogic\Rhapsody\Common"

if %ARCH%==32bit (
	if %OS_ARCH%==x64 (
		set COMMONREGKEY="HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Telelogic\Rhapsody\Common"
	)
)
rem echo Rhapsody Common registry key is "%COMMONREGKEY%"

rem get registry values for rhapsody TLB path
set REGVALUE="TypeLibPath"
For /F "Tokens=2*" %%A In ('%SystemRoot%\System32\reg query "%COMMONREGKEY%" /v %REGVALUE%  ^| %SystemRoot%\System32\Find /I  %REGVALUE% ') do Call Set OLD_TLBPATH=%%B 
echo Current Rhapsody TLB path in registry is:
echo 	"%OLD_TLBPATH%"

rem substitute with the new value
set "NEW_TLBPATH=!OLD_TLBPATH:Rhapsody2.tlb=IBM\Rational\Rhapsody\rhapsody2.tlb!"

echo "%OLD_TLBPATH%" | %SystemRoot%\System32\FindSTR /C:"Rational" >nul & IF  ERRORLEVEL 1 (
	echo Updated Rhapsody TLB Path:
	echo 	"%NEW_TLBPATH%"
) else (
	echo TLB Path does not need any update.
)

rem get registry values for rhapsody 831 executable
set REGKEY="HKEY_LOCAL_MACHINE\SOFTWARE\Telelogic\Rhapsody\%VER%"
if %ARCH%==32bit (
	if %OS_ARCH%==x64 (
		set REGKEY="HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Telelogic\Rhapsody\%VER%"
	)
)

set REGVALUE="InstallationDirectory"
For /F "Tokens=2*" %%A In ('%SystemRoot%\System32\reg query "%REGKEY%" /v %REGVALUE%  ^| %SystemRoot%\System32\Find /I  %REGVALUE% ') do Call Set RHPPath=%%B
echo Rhapsody path is:
echo 	"%RHPPath%"

if %OS_ARCH%==x64 (
	set REGKEY="HKEY_LOCAL_MACHINE\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion"
	) else (
	set REGKEY="HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion"
)

rem detecting common files dir
set REGVALUE="CommonFilesDir"
For /F "Tokens=2*" %%A In ('%SystemRoot%\System32\reg query "%REGKEY%" /v %REGVALUE%  ^| %SystemRoot%\System32\Find /I  %REGVALUE% ') do Call Set COMMON_FILES_DIR=%%B
echo Common files directory of 32 bit is:
echo 	"%COMMON_FILES_DIR%"

rem prevent corrupting the path by running again and again
ECHO "%OLD_TLBPATH%" | %SystemRoot%\System32\FindSTR /C:"Rational" >nul & IF ERRORLEVEL 1 (
	goto update_registry_key 
	) else (
	goto run_rhapsody_regserver
)


rem create common files rhapsody dir
if not exist "%COMMON_FILES_DIR%\IBM\" md "%COMMON_FILES_DIR%\IBM"
if not exist "%COMMON_FILES_DIR%\IBM\Rational\" md "%COMMON_FILES_DIR%\IBM\Rational"
if not exist "%COMMON_FILES_DIR%\IBM\Rational\Rhapsody\" md "%COMMON_FILES_DIR%\IBM\Rational\Rhapsody"

if not exist "%COMMON_FILES_DIR%\IBM\Rational\Rhapsody" (
	echo Error failed to create directory "%COMMON_FILES_DIR%\IBM\Rational\Rhapsody"
	goto finish
)
	
:update_registry_key
set REGVALUE="TypeLibPath"
%SystemRoot%\System32\reg add "%COMMONREGKEY%" /v "%REGVALUE%" /f  /d "%NEW_TLBPATH%"
if errorlevel 0 (
	echo Registry TLB path updated successfuly
	goto run_rhapsody_regserver
	) else (
		echo Error updating TLB path!
		goto finish
	)

:run_rhapsody_regserver
call "%RHPPath%Rhapsody.exe" /RegServer
if errorlevel 0 (
	if errorlevel 0 echo Rhapsody updated registry
	if exist "%NEW_TLBPATH%" (
		if not "%NEW_TLBPATH%"=="%OLD_TLBPATH%" (
			if exist "%OLD_TLBPATH%" (
				echo deleting old TLB from "%OLD_TLBPATH%"
				del "%OLD_TLBPATH%"
			)
		)
	)
) else (
	echo Error registring Rhapsody
)



:finish
if defined FromInstaller pause
ENDLOCAL 
echo ************ Rhapsody iFix Post Installation Finished *****************